﻿Function New-RandomPassword {
<#
.SYNOPSIS
    This advanced function ...

.DESCRIPTION
    ...

.PARAMETER <ParameterName>
    The <ParameterName> parameter ...

.EXAMPLE
    This example ...a
    PS > <Example>
    
.NOTES
    Name: New-RandomPassword
    Author: <Name>
    Comments: --
    Last Edit: xx/xx/xxxx [1.0]
    Template Version 1.2.3
    Version 1.0
        - <VersionNotes> (or delete if no version notes)
#>
    [CmdletBinding()]
    Param (
        [Parameter()]
        [ValidateRange(4,35)]
        [int]$Length = 10,

        [Parameter()]
        [ValidateSet('Number','Uppercase','Lowercase','Symbol')]
        [string[]]$CharacterType = ('Number','Uppercase','Lowercase','Symbol'),

        [Parameter()]
        [string[]]$UnusedCharacter
    )

    DynamicParam {
        # Create dynamic, Log parameter.
        If ($PSBoundParameters['Verbose']) {
            $SingleAttribute = New-Object System.Management.Automation.ParameterAttribute
            $SingleAttribute.Position = 1

            $AttributeCollection = New-Object System.Collections.ObjectModel.Collection[System.Attribute]
            $AttributeCollection.Add($SingleAttribute)

            $LogParameter = New-Object System.Management.Automation.RuntimeDefinedParameter('Log',[switch],$AttributeCollection)
            $LogParameter.Value = $true
 
            $ParamDictionary = New-Object System.Management.Automation.RuntimeDefinedParameterDictionary
            $ParamDictionary.Add('Log',$LogParameter)
            return $ParamDictionary
        } # End If.
    } # End DynamicParam.

    Begin {
        #region Create logs directory and Write-Verbose function.
        If ($PSBoundParameters['Verbose'] -and $PSBoundParameters['Log']) {
            $LogDirectory = "$($MyInvocation.MyCommand.Name)"
            $LogPath = "$env:SystemDrive\support\Logs\$LogDirectory"
            If (-Not(Test-Path -Path $LogPath)) {
                [System.Void](New-Item -Path $LogPath -ItemType Directory)
            }
            $LogFilePath = "$LogPath\$(Get-Date -Format 'DyyyyMMddThhmmsstt').txt"

            Function Write-Verbose {
                Param ($Message)
                Microsoft.PowerShell.Utility\Write-Verbose -Message $Message
                Microsoft.PowerShell.Utility\Write-Verbose -Message "[$(Get-Date -Format G)]: $Message" 4>> $LogFilePath
            }
        }

        # Set Write-Verbose block location.
        $BlockLocation = '[BEGIN  ]'
        Write-Verbose -Message "$BlockLocation Entering the Begin block [Function: $($MyInvocation.MyCommand.Name)]."
        #endregion
    } # End Begin.

    Process {
        #region Set Write-Verbose block location.
        $BlockLocation = '[PROCESS]'
        Write-Verbose -Message "$BlockLocation Entering the Process block [Function: $($MyInvocation.MyCommand.Name)]."
        #endregion

        #region Create character set.
        Write-Verbose -Message "$BlockLocation Creating a random password with $Length characters."
        Write-Verbose -Message "$BlockLocation Building the allowable character set."
        Foreach ($Character in $CharacterType) {
            Switch ($Character) {
                'Number' {
                    # Numbers.
                    For ($a = 48; $a -le 57; $a++) {
                        $CharacterSet += [array][char]$a
                    }
                }
                'Uppercase' {
                    # Uppercase.
                    For ($b = 65; $b -le 89; $b++) {
                        $CharacterSet += [array][char]$b
                    }
                }
                'Lowercase' {
                    # Lowercase.
                    For ($c = 97; $c -le 122; $c++) {
                        $CharacterSet += [array][char]$c
                    }
                }
                'Symbol' {
                    # Symbols.
                    For ($d = 33; $d -le 47; $d++) {
                        $CharacterSet += [array][char]$d
                    }
                    For ($d = 58; $d -le 64; $d++) {
                        $CharacterSet += [array][char]$d
                    }
                    For ($d = 91; $d -le 96; $d++) {
                        $CharacterSet += [array][char]$d
                    }
                    For ($d = 123; $d -le 126; $d++) {
                        $CharacterSet += [array][char]$d
                    }    
                }
            } # End Switch.
        } # End Foreach.
        Write-Verbose -Message "$BlockLocation Displaying the allowable characters set as $($CharacterSet -join ',')."
        #endregion

        #region Remove unwanted characters.
        If ($UnusedCharacter) {
            Write-Verbose -Message "$BlockLocation Removing unwanted character(s) $($UnusedCharacter -join ',') from the character set."

            $CharacterSet = {$CharacterSet}.Invoke()
            Foreach ($Character in $UnusedCharacter) {
                [System.Void]($CharacterSet.Remove($Character))
            }
            Write-Verbose -Message "$BlockLocation Displaying the updated allowable characters set as $($CharacterSet -join ',')."
        } # End If.
        #endregion                                                                              s

        #region Create random password.
        Write-Verbose -Message "$BlockLocation Creating the random password."
        For ($Loop = 1; $Loop –le $Length; $Loop++) {
            $RandomPassword += ($CharacterSet | Get-Random)
        } # End For.
        #endregion.
    } # End Process.

    End {
        #region Set Write-Verbose block location.
        $BlockLocation = '[END    ]'
        Write-Verbose -Message "$BlockLocation Entering the End block [Function: $($MyInvocation.MyCommand.Name)]."
        #endregion

        #region Display random password.
        Write-Verbose -Message "$BlockLocation Displaying the random password."
        $RandomPassword
        #endregion
    } # End End.
} # End Function: New-RandomPassword.

# The following commands are in place for the Arizona PowerShell Saturday 2017.
$env:USERDOMAIN = 'MYDOMAIN'
$env:USERNAME = 'tommymaynard'
$env:COMPUTERNAME = 'PSSATURDAYPC'
Remove-Item -Path C:\support* -Recurse -ErrorAction SilentlyContinue
$PSDefaultParameterValues.Clear()
Clear-Host